#include <dcc_std.h>
#include <dcc_misc.h>

#include <unistd.h>

#include <errno.h>

#include <signal.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/uio.h>

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#include <pthread.h>

#include "forker.h"

extern int receiver_creates;
extern int client_creates;

/*

   reporttime network station server-address server-port start-time
   target-address target-port connect-flag last-good-con-time last-good-rec-rm #recs-rec
   retrain# connects rec-this-con receiver-state trouble num error-message
   chank last-time rec-time #recsincestart #rectoday(gmt)
   receiver_creates client_creates
   /
   client# client-addr connect-time time-last-sent last-seq recs-trxfred
   /


*/

/* from forkmain */

extern char *rec_hostname;
extern int rec_port;
extern int rec_size;
extern int rec_buff;
extern int con_port[MAX_CONN_PORTS];
extern char *network;
extern char *station;
extern time_t startup;

/* from fork_receiver */

extern int receiverconnected;
extern char *receiverstatus;
extern int retrain;
extern int connects;
extern int seqno;
extern long rectotal;
extern time_t last_connect;
extern time_t last_record;
extern int last_rec_error;
extern int receiverintrouble;

#define MAX_CONNECTORS 64 /* Somewhere here we exceed max files */

extern struct _client_table cli_table[MAX_CONNECTORS];

void memstatus()
{

  char *setup;
  char file1[100];
  char file2[100];
  FILE *outf;
#ifdef HAVE_MALLINFO
  struct mallinfo rmal;
#endif

  setup = getenv("LISS_1LOGS");
#ifdef LISSSTATUS
  if (setup==NULL)
    setup = LISSSTATUS;
#endif
  if (setup) {
    sprintf(file1,"%s/forker_%s_%s.mem.tmp",
	    setup,network,station);
    sprintf(file2,"%s/forker_%s_%s.memdump",
	    setup,network,station);
  } else {
    
    setup = getenv("DCC_SETUPS");
    if (setup==NULL) setup="/tmp";
    
    sprintf(file1,"%s/tel/forker_%s_%s.mem.tmp",
	    setup,network,station);
    sprintf(file2,"%s/tel/forker_%s_%s.memdump",
	    setup,network,station);
  }
  
  outf = fopen(file1,"w");
  if (outf==NULL) return;
  
#ifdef HAVE_MALLINFO
  rmal = mallinfo();
  
  fprintf(outf,"arena=%d ordblks=%d smblks=%d hblkhd=%d hblks=%d usmblks=%d fsmblks=%d uordblks=%d fordblks=%d keepcost=%d\n",
	  rmal.arena,
	  rmal.ordblks,
	  rmal.smblks,
	  rmal.hblkhd,
	  rmal.hblks,
	  rmal.usmblks,
	  rmal.fsmblks,
	  rmal.uordblks,
	  rmal.fordblks,
	  rmal.keepcost);
#endif
  
  fclose(outf);
  
  unlink(file2);
  link(file1,file2);
  unlink(file1);
  
  
}

void *Start_Status_Dumper(void *inval)
{

  char *setup;
  char file1[100];
  char file2[100];
  FILE *outf;
  int i;

  setup = getenv("LISS_1LOGS");
  if (setup) {
    sprintf(file1,"%s/forker_%s_%s.tmp",
	    setup,network,station);
    sprintf(file2,"%s/forker_%s_%s.status",
	    setup,network,station);
  } else {
    
    setup = getenv("DCC_SETUPS");
    if (setup==NULL) setup="/tmp";
    
    sprintf(file1,"%s/tel/forker_%s_%s.tmp",
	    setup,network,station);
    sprintf(file2,"%s/tel/forker_%s_%s.status",
	    setup,network,station);
  }
  
  FOREVER {

    outf = fopen(file1,"w");
    if (outf==NULL) {
      sleep(60);
      continue;
    }

    /* GTM 12-feb-02 defer adding new con_ports to output until revforker modified */
    fprintf(outf,"%ld %s %s unknown %d %ld %ld %ld %ld\n",
	    (long) time(NULL), 
	    network,
	    station,
	    con_port[0], 
	    (long) startup,
	    (long) getpid(),
	    (long) getpgrp(),
	    (long) getppid());

    fprintf(outf,"%s %d %d %ld %ld %ld\n",
	    rec_hostname,
	    rec_port,
	    receiverconnected,
	    (long) last_connect,
	    (long) last_record,
	    rectotal);

    fprintf(outf,"%d %d %d %s %d %d %s\n",
	    retrain,
	    connects,
	    seqno,
	    receiverstatus,
	    receiverintrouble,
	    last_rec_error,
	    strerror(last_rec_error));

    fprintf(outf,"%d %d\n",receiver_creates,client_creates);

    fprintf(outf,"/\n");

    for (i=0; i<MAX_CONNECTORS; i++) {

      unsigned char *ad_data;

      if (cli_table[i].client_num<0) continue;
      
      ad_data = (unsigned char *) &cli_table[i].cli_addr.sin_addr;

      fprintf(outf,"%d %d.%d.%d.%d:%d ",
	      cli_table[i].conseq,
	      ad_data[0],
	      ad_data[1],
	      ad_data[2],
	      ad_data[3],
	      htons(cli_table[i].cli_addr.sin_port));

      fprintf(outf,"%ld %ld %d %ld\n",
	      cli_table[i].started,
	      cli_table[i].lastgood,
	      cli_table[i].sent_seq,
	      cli_table[i].reccopy);

    }

    fprintf(outf,"/\n");

    fclose(outf);

    unlink(file2);
    link(file1,file2);
    unlink(file1);

    memstatus();

    sleep(20);
    sched_yield();

  }

  return(NULL);

}

