#include <dcc_std.h>
#include <dcc_misc.h>

#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/uio.h>

#include <pthread.h>
#include <signal.h>

#include "forker.h"

extern char *bindaddress;
extern char *rec_hostname;
extern int rec_port;
extern int rec_size;
extern int rec_buff;

extern int con_port[MAX_CONN_PORTS];  /* GTM 12-feb-02 changed con_port to array */
extern int num_conn_ports;

struct hostent bindaddr,*bindret;
char bindbuff[512];

struct _client_table cli_table[MAX_CONNECTORS];

void *Fork_Client(void *);

/* This block protected by RW lock ringlock */

extern pthread_rdwr_t ringlock;

extern char *seedbuf1;
extern char *masterring;

struct _rectraf {
  time_t recrec;
  int intseq;
  int orgseq;
};

int client_creates = 0;

extern struct _rectraf *rectraf;

extern int ring_point;

/* End of ringlock block */

extern int seqno;

void *Start_Connector(void *inptr)
{
  /* GTM 12-feb-02 changed listen_socket to array just like con_port */
  int listen_socket[MAX_CONN_PORTS]; 
  int sockindex;

  int i;
  int attempts = 0;

  struct sockaddr_in listen_addr; /* Listen address setup */

  for (i=0; i<MAX_CONNECTORS; i++)
    cli_table[i].client_num = -1;

  /* Create the sockets for listening to clients requesting data  */
  /* GTM 12-feb-02 turned this whole block of code into a loop to set up an array
     of sockets  */
  for (sockindex = 0; sockindex < num_conn_ports; sockindex++) {

     listen_socket[sockindex] = socket(AF_INET, SOCK_STREAM, 0); /* TCP */
     if (listen_socket[sockindex]<0) {
        fprintf(stderr,"dnetport: Cannot open socket (err %d,%d)\n", errno,
                listen_socket[sockindex]);
        return(NULL);
     }

     /* Set up structure and bind to port number */
     /* Add code to bind to a specific address (virtual IP) */
     listen_addr.sin_addr.s_addr = INADDR_ANY; /* Any can connect */

     if (bindaddress) {
        int rerr=0;
#ifdef HAVE_GETHOSTBYNAME_R  /* IGD 01/17/02 */
        bindret = gethostbyname_r(bindaddress,  &bindaddr, bindbuff,sizeof(bindbuff), &rerr);
#else
        bindret = gethostbyname(bindaddress);
#endif

        /* FATAL EXIT */
        if (bindret==NULL) {
           fprintf(stderr,"{Cannot find host IP you wanted me to bind to: %s %d %s}\n",
	           bindaddress,rerr,strerror(errno));
           exit(10);
        }

        endhostent();

        memcpy((void *) &listen_addr.sin_addr, (void *) bindret->h_addr, 
               (size_t) bindret->h_length);

     } else {
        fprintf(stderr,"{No bindaddress set. Using INADDR_ANY for bind address}\n");
     }

     listen_addr.sin_family = AF_INET;
     listen_addr.sin_port = htons(con_port[sockindex]); /* Byte swapping */
     fprintf (stderr, "{Attempting to bind %s:%d socket = %d}\n", 
	      bindaddress,   listen_addr.sin_port, listen_socket[sockindex]); 

     /* allow local port reuse */
     { int opt = 1; 
     if (setsockopt(listen_socket[sockindex], SOL_SOCKET, SO_REUSEADDR, 
         (char *)&opt, sizeof(opt))<0) {
         perror("setsockopt: SO_REUSEPORT)");
         (void)close(listen_socket[sockindex]);
         return(NULL);
     } /* if */
     }

     /* don't wait for data to transmit on close */

     { struct linger ld; 
     ld.l_onoff=0;
     ld.l_linger=0;
     if (setsockopt(listen_socket[sockindex], SOL_SOCKET, SO_LINGER, 
         (char *)&ld, sizeof(ld))<0) {
        perror("create_socket(setsockopt: SO_LINGER)");
        (void)close(listen_socket[sockindex]);
        return(NULL);
     } /* if */
     }


     sleep(2);

     FOREVER {
        if (bind(listen_socket[sockindex], (struct sockaddr *) &listen_addr,
            sizeof(listen_addr)) < 0) {
           fprintf(stderr, "{ Bind failed %s %d}\n", bindaddress, listen_addr.sin_port);
           if (errno==EADDRINUSE) {
              fprintf(stderr,"Port %d already in use - is an instance of this program already running?\n", 
                      con_port[sockindex]);
              sleep(60);
              continue;
           }
           if (errno==EBADF)  {
              fprintf (stderr, "{Failed binding on port %d: bad file number; try again}\n", 
                       con_port[sockindex]);
              sleep(20); 
              attempts++;
              if (attempts > 20) /* IGD 01/22/02 */
                 exit(11);
              continue;
           }

           fprintf(stderr,"[Cannot bind (err %d - %s)]\n",errno,strerror(errno));
           exit(10);
        }  

        break;
        attempts = 0;
     }  /* end FOREVER */

     if (listen(listen_socket[sockindex], PEND_DEPTH) < 0) {
       fprintf(stderr,"dnetport: cannot set depth to %d (%d)\n", PEND_DEPTH,errno);
       exit(10);
     }

  }  /* end of for(sockindex ... */


  FOREVER {  /* Loop forever waiting for connection requests */

    int client_sock;
    struct sockaddr_in addr;
    int addr_len = sizeof(struct sockaddr_in);
    int client;
    int erval;
    sigset_t new_mask, orig_mask;

    pthread_attr_t attr;
    int sockindex;  
    int numselected;
    int largest_fd;
    fd_set readmask;  


    /* GTM 13-feb-02  setup the readmask that will be used in the call to select */
    FD_ZERO(&readmask);
    largest_fd = -1;
    for (sockindex = 0; sockindex < num_conn_ports; sockindex++) {
       FD_SET(listen_socket[sockindex], &readmask);
       if (listen_socket[sockindex] > largest_fd) {
         largest_fd = listen_socket[sockindex];
       }
    }
    largest_fd += 1;

    /* use select to effectively poll all of the listen sockets */
    numselected = select(largest_fd, &readmask, NULL, NULL, NULL);
    if (numselected == -1) {
      fprintf(stderr, "select failed, returned %d", numselected);
      continue;
    }

    /* determine which listen socket got the connect */ 
    for (sockindex = 0; sockindex < num_conn_ports; sockindex++) {
      if (FD_ISSET(listen_socket[sockindex], &readmask)) {
         break; 
      }
    }  

    client_sock = accept(listen_socket[sockindex], (struct sockaddr *) &addr, &addr_len);
    if (client_sock < 0) {
       fprintf(stderr,"accept failed (errno:%d,client:%d,socket:%d,index:%d)\n", 
               errno, client_sock, listen_socket[sockindex], sockindex);
       sleep(10);
       continue;
    }
 
    fprintf (stderr, "[Connection accepted on %d]\n", client_sock);

    if (getpeername(client_sock, (struct sockaddr *) &addr,&addr_len) < 0) {
       fprintf(stderr,"BAD ERROR encountered - error %d - %s\n", errno, strerror(errno));
       close(client_sock);
       sleep(20);
       continue;
    }

    client = -1;
    for (i=0; i<MAX_CONNECTORS; i++) 
      if (cli_table[i].client_num<0) client = i;

    if (client<0) {  /* No free slots */
      close(client_sock);
      sleep(60);
      continue;
    }

    /* GTM 12-feb-02  use cli_table.conn_req_type to let fork_client know which port
       got connected so it can filter data accordingly */
    cli_table[client].conn_req_type = sockindex; 
    cli_table[client].client_socket = client_sock;
    cli_table[client].cli_addr_len = addr_len;
    memcpy((void *) &cli_table[client].cli_addr, (void *) &addr, 
            sizeof(struct sockaddr_in));
    cli_table[client].client_num = client;
    cli_table[client].sent_seq = seqno;
    cli_table[client].sent_seq_index = ring_point;
    cli_table[client].lastgood = 0;
    cli_table[client].reccopy = 0;

    /* create the process thread to handle this client */

    (void) sigfillset(&new_mask);
    (void) sigdelset(&new_mask, SIGINT);
    (void) pthread_sigmask(SIG_SETMASK, &new_mask, &orig_mask);

    cli_table[client].client_tid = 0;

    cli_table[client].started = time(NULL);
    cli_table[client].conseq = client_creates+1;

    /* GTM 14-feb-02 fix memory leak */
    pthread_attr_init( &attr);
    pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED);

    erval = pthread_create(&cli_table[client].client_tid,
			   &attr,
			   Fork_Client,
			   (void *) &cli_table[client]);
    if (erval) {
      perror("pthread_create");
      fprintf(stderr,"Could not spawn the client thread (%d,%d,%d)\n",
	      erval,errno,(int) cli_table[client].client_tid);
      close(client_sock);
      sleep(60);
      continue;
    }

    client_creates ++;

    cli_table[client].lastgood = 0;

    (void) pthread_sigmask(SIG_SETMASK, NULL, &orig_mask);

    sched_yield();

  }
}
