/* Filename: fork_receiver.c
 * Modifications:
 *   July 18, 2003 - reset timeout variable when non-data record received
 */
#include <dcc_std.h>
#include <dcc_misc.h>

#include "forker.h"

#define MAX_RETRAINS 10000

extern char *rec_hostname;
extern int rec_port;
extern int rec_size;
extern int rec_buff;

extern char *network;
extern char *station;

#define WATCHDOG (60*5)

char *receiverstatus = "UNSET";

int receiverconnected = 0;
int receivervalid = 0;
int receiverintrouble = 0;
int activereceiver = 0;
time_t rec_watchdog = 0;
pthread_t active_receiver_tid=0;
int last_rec_error = 0;

int non_data_record=0;
#define NON_DATA_MAX 50

int receiver_creates = 0;

/* This block protected by RW lock ringlock */

pthread_rdwr_t ringlock;

char *seedbuf1=NULL;

struct _rectraf {
  time_t recrec;
  int intseq;
  int orgseq;
  int ident;
  char *recbuf;
} *rectraf;

int ring_point= 0;

/* End of ringlock block */

int seqno=0;
long rectotal=0;
int retrain=0;
int connects=0;
time_t last_connect = 0;
time_t last_record = 0;

void albufs()
{

  int i;

  if (seedbuf1) return;

  pthread_rdwr_init_np(&ringlock, NULL);

  pthread_rdwr_wlock_np(&ringlock);

  seedbuf1 = (char *) malloc(rec_size*2);
  if (seedbuf1==NULL)
    bombout(EXIT_INSFMEM,"Couldn't alloc buffers");

  rectraf = (struct _rectraf *) malloc(sizeof(struct _rectraf)*rec_buff);
  if (rectraf==NULL) 
    bombout(EXIT_INSFMEM,"Couldn't alloc buffers");

  for (i=0; i<rec_buff; i++) {
    rectraf[i].recrec = 0;
    rectraf[i].intseq = -1;
    rectraf[i].orgseq = -1;
    rectraf[i].ident = 0;
    rectraf[i].recbuf = (char *) malloc(rec_size);
  }

  pthread_rdwr_wunlock_np(&ringlock);

}

void procdata() {

  receivervalid = 1;
  receiverstatus = "NORMAL";
  last_record = time(NULL);


  /* The following will need to change when data contains other record
     types besides data - but then each possible value should be checked */

  if (seedbuf1[6] != 'D' && seedbuf1[6] != 'Q' && seedbuf1[6] != 'R') {
    non_data_record++;
    rec_watchdog = time(NULL);
    return;
  }

  non_data_record = 0;

  /* Set the network code if it is not set (14-Aug-97/SH) */

  if (!isalnum((int) seedbuf1[18])) {
    seedbuf1[18] = ' ';
    seedbuf1[19] = ' ';
    strncpy(&seedbuf1[18],network,strlen(network));
  }

  pthread_rdwr_wlock_np(&ringlock);

  ring_point++;
  ring_point %= rec_buff;

  memcpy((void *) rectraf[ring_point].recbuf, seedbuf1, rec_size);
  
  rectraf[ring_point].recrec = time(NULL);
  rectraf[ring_point].intseq = ++seqno;
  rectraf[ring_point].orgseq = 0;
  rectraf[ring_point].ident = (ring_point + 5) * (ring_point + 10);

  rectotal++;

#ifdef DEBUG
  fprintf (stderr, "[MSEED from %s:%d %c%c%c %c%c %d records]\n",
	rec_hostname,
	rec_port,
	(seedbuf1[8]),
	(seedbuf1[9]),
	(seedbuf1[10]),
	(seedbuf1[18]),
	(seedbuf1[19]),
	rectotal);
#endif	
  pthread_rdwr_wunlock_np(&ringlock);

#if 0
  printf("%d, %d\r",seqno,ring_point);
  fflush(stdout);
  if (ring_point>seqno)
    printf("seq=%d < ring=%d ***********************************\n",
	   seqno,ring_point);
#endif
	 
  rec_watchdog = time(NULL);
  receiverintrouble=0;

}

int recsocket= -1;

int make_con()
{
  
  struct hostent *hostret;
#ifdef HAVE_GETHOSTBYNAME_R
  struct hostent hostaddr;
  char stbuff[512];
#endif
  int reterror;
  struct sockaddr_in sockin;

  /*
   * Translate the hostname into an internet address
   */



  retrain++;
  if (retrain>MAX_RETRAINS) {
    printf("Maximum number retrains exceeded\n");
    exit(0);
  }

  receiverstatus = "CONNECT1";

  reterror = 0;

#ifdef HAVE_GETHOSTBYNAME_R
  hostret = gethostbyname_r(rec_hostname, 
			  &hostaddr,
			  stbuff, sizeof(stbuff),
			  &reterror);
#else
  hostret = gethostbyname(rec_hostname);
#endif

  if(hostret==NULL) {
    last_rec_error = errno;
    fprintf(stderr, "%s: unknown host error=%d\n",rec_hostname,reterror);
    return(0);
  }

  endhostent();

  /*
   * We now create a socket which will be used for our connection.
   */

  receiverstatus = "CONNECT2";

  recsocket = socket(AF_INET, SOCK_STREAM, 0);
  if(recsocket < 0) {
    last_rec_error = errno;
    perror("Error opening socket");
    return(0);
  }
  
  printf("[Socket id is %d]\n",recsocket);

  /*
   *  Before we attempt a connect, we fill up the "sockaddr_in" structure
   */

  memcpy((void *) &sockin.sin_addr,
	 (void *) hostret->h_addr,
	 (size_t) hostret->h_length);

  sockin.sin_family      = AF_INET;
  sockin.sin_port        = htons(rec_port);

  /*
   *  Now attempt to connect to the remote host .....
   */

  receiverstatus = "CONNECT3";

  if(connect(recsocket, 
	     (struct sockaddr *) &sockin, 
	     sizeof(sockin)) < 0) {
    last_rec_error = errno;
    fprintf(stderr, "[Addr: %s Port: %d - ",rec_hostname,rec_port);
    fflush(stdout);fflush(stderr);
    perror("Unable to make connection");
    fprintf(stderr, "]\n");
    close(recsocket);
    return(0);
  }
    fprintf(stderr, "[ Connected to DA: Addr: %s Port: %d]\n",
	rec_hostname,rec_port);
  connects++;

  syslog(LOG_NOTICE,
	 "[SOURCE up: %s (%d) Conn=%d/Ret=%d]\n",
	 hostret->h_name,rec_port,connects,retrain);

  receiverconnected = 1;
  receiverstatus = "CONNECTED";
  last_connect = time(NULL);
  last_rec_error = 0;

  rec_watchdog = time(NULL);
  receiverintrouble=0;
  non_data_record=0;

  return(recsocket);

}

#define TRYMAX 4

void copypackets()
{

  int bufpop = 0;
  int i;
  int optval;

  /* Set the recsocket to non-blocking so we can detect when
     the link has gone insane */

  i = fcntl(recsocket, F_SETFL, O_NONBLOCK); 
  if (i==(-1)) {
    printf("Couldn't mark receiver socket non-blocking\n");
    exit(10);
  }

  optval = 1;
  i = setsockopt(recsocket, SOL_SOCKET, 
		 SO_KEEPALIVE, 
		 (char *) &optval, sizeof(int));

  if (i==(-1)) 
    printf("Couldn't set keepalive on receiver socket (%s)\n",
	   strerror(errno));

  sleep(1);
  for (;;) {
    int count;
	
    count = recv(recsocket, &seedbuf1[bufpop], 256, 0);

    /* IGD 01/23/02 EBADF is one more possible condition */
    if (count==(-1) &&
	(errno==EWOULDBLOCK || errno==EINTR || errno==EBADF)) {
//	syslog(LOG_NOTICE,"[RECV FAILED: errno = %d socket = %d]\n",
//	       errno, recsocket);
      if (receiverintrouble) {
	syslog(LOG_NOTICE,"[SOURCE down: timeout (no data for %d secs)]\n",
	       WATCHDOG);
	activereceiver = 0;
	receiverconnected = 0;
	receiverstatus = "TIMEOUT";
	active_receiver_tid = 0;
	close(recsocket);
	pthread_exit(NULL);
      }
      if (non_data_record>NON_DATA_MAX) {
	syslog(LOG_NOTICE,"[SOURCE down: Too many non-data records seen - unsyncronized]\n");
	activereceiver = 0;
	receiverconnected = 0;
	receiverstatus = "UNSYNC";
	active_receiver_tid = 0;
	close(recsocket);
	pthread_exit(NULL);
      }
      sched_yield();
      continue;
    }

    if (count<=0) {
      syslog(LOG_NOTICE,"[SOURCE down: read %d (err=%d,%s)]\n",
	     count,errno,strerror(errno));
      activereceiver = 0;
      receiverconnected = 0;
      receiverstatus = "SOCKET";
      active_receiver_tid = 0;
      close(recsocket);
      pthread_exit(NULL);
    }

    bufpop += count;

    /* Drain buffer (15-May-98/SH) */

    while (bufpop>=rec_size) {

      procdata();
      bufpop -= rec_size;

      if (bufpop>0)
	memcpy(&seedbuf1[0],&seedbuf1[rec_size], bufpop);

    }

  }

}

void *Start_Active_Receiver(void *inptr)
{

  int recsocket=0;
  int tries=0;

  receiverstatus = "STARTING";
  receiverconnected = 0;
  activereceiver = 1;

  for (tries=0; tries<TRYMAX; tries++) {
    recsocket = make_con();
    if (recsocket) break;
    receiverstatus = "RETRYING";
    sleep(10);
  }

  if (!recsocket) {
    activereceiver = 0;
    receiverconnected = 0;
    receiverstatus = "UNSUCESSFUL";
    active_receiver_tid = 0;
    printf("We give up - kill receiver\n");
    pthread_exit(NULL);
  }

  copypackets();

  pthread_exit(NULL);  /* Just in case it gets here */

  return(NULL);

}

void startreceiver()
{

  int erval;
  sigset_t new_mask, orig_mask;
  pthread_attr_t attr;

  activereceiver = 0;
  active_receiver_tid = 0;

  printf("Start Receiver\n");

  receiverstatus = "FORKING";

  (void) sigfillset(&new_mask);
  (void) sigdelset(&new_mask, SIGINT);
  (void) pthread_sigmask(SIG_SETMASK, &new_mask, &orig_mask);

  /* GTM 14-feb-02 fix memory leak */
  pthread_attr_init( &attr);
  pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED);

  erval = pthread_create(&active_receiver_tid, 
			 &attr, 
			 Start_Active_Receiver, 
			 NULL);
  if (erval) {
    perror("pthread_create");
    bombout(erval,"Could not spawn the active receiver thread (%d,%d,%d)\n",
	    erval,errno,active_receiver_tid);
  }

  receiver_creates ++;

  (void) pthread_sigmask(SIG_SETMASK, NULL, &orig_mask);

}

/* Receiver controller/watchdog */

void *Fork_Receiver(void *inptr)
{

  time_t now;

  receiverstatus = "BEGIN";

  if (seedbuf1==NULL) albufs();

  FOREVER {

    /* If there's no receiver, start one */

    if (!activereceiver) {
      startreceiver();
      sleep(60);
      continue;
    }

    now = time(NULL);

    if ((now - rec_watchdog) > WATCHDOG) {
      receiverintrouble=1;
    }

    sleep(10);
    sched_yield();

  }

  pthread_exit(NULL);
}

