#include <dcc_std.h>
#include <dcc_misc.h>

/*

reporttime network station server-address server-port start-time pid pgid ppid
target-address target-port connect-flag last-good-con-time last-good-rec-rm #recs-rec
retrain# connects rec-this-con receiver-state trouble num error-message
chank last-time rec-time #recsincestart #rectoday(gmt)
/
client-addr connect-time time-last-sent recs-trxfred
/


Station  Uptime   Status                      Retrain  Last Rec   Recs   Conn
IU ANMO xxx,xx:xx CONNECTED                   xxx/xxx xxx,xx:xx xxxxxx xxxxxx
                  CONNECT3 Error Text        

*/

#include <dirent.h>
#include <errno.h>

#define MAXSTAT 1000

char *statfiles[MAXSTAT];
int statnum;

int webmode=0;

int argcnt=0;
char *arglist[11];
int mode;

int compstr(const char **s1, const char **s2)
{

  return(strcmp(*s1,*s2));
}

void setup_args()
{

  char inargs[2048],*w,*o;

  argcnt = 0;
  mode = 0;

  w = getenv("PATH_INFO");
  
  if (w) {

    strcpy(inargs,w);
    Upcase(inargs);

    for (w=NonWhite(inargs),o=strtok(w,"/"); o!=NULL; o=strtok(NULL,"/")) 
      if (argcnt<10)
	arglist[argcnt++] = strdup(o);

    if (argcnt>0) 
      mode = atoi(arglist[0]);
  }

}


char _buf[40];

char *uptime(time_t rtime)
{

  time_t now = time(NULL);
  int delta,hour=0,min=0,day=0;


  if (webmode && rtime==0) 
    return(" <blink> Never </blink>  ");

  if (rtime==0) return("  Never  ");

  delta = (int) now - rtime;
  
  if (delta<60) return("  Recent ");

  min = delta / 60;
  if (min > 59) {
    hour = min / 60;
    min %= 60;
  }

  if (hour>23) {
    day = hour / 24;
    hour %= 24;
  }

  sprintf(_buf,"%3d,%02d:%02d",day,hour,min);

  return(_buf);
}

int interv(time_t rtime)
{

  time_t now = time(NULL);
  time_t delta;

  delta = (int) now - rtime;
  
  return((int) delta);

}

void process_stats(char *inname)
{

  FILE *inf;
  int xper=0;

  char work1[100];
  char *statball = "Red.gif";

  time_t lastreport,startup;
  char network[20],station[20],ipaddress[200];
  int con_port;
  pid_t pid,pgrp,ppid;

  char hostname[200];
  int rec_port,receiverconnected;
  time_t last_connect,last_record;
  long rectotal;

  int retrain,connects,seqno,receiverintrouble;
  int last_rec_error;
  char status[100];

  inf = fopen(inname,"r");
  if (inf==NULL) {
    printf("Cannot open status file %s\n",inname);
    return;
  }

  fscanf(inf,"%d %s %s %s %d %d %d %d %d\n",
	 (int *) &lastreport,
	 network,
	 station,
	 ipaddress,
	 &con_port,
	 (int *) &startup,
	 (int *) &pid,
	 (int *) &pgrp,
	 (int *) &ppid);
  
  fscanf(inf,"%s %d %d %d %d %ld",
	 hostname,
	 &rec_port,
	 &receiverconnected,
	 (int *) &last_connect,
	 (int *) &last_record,
	 &rectotal);
  
  fscanf(inf,"%d %d %d %s %d %d",
	 &retrain,
	 &connects,
	 &seqno,
	 status,
	 &receiverintrouble,
	 &last_rec_error);

#define YELLOWINT (24*3600)   /* Up in last day */
#define BLUEINT (4*3600)  /* 4 hours gets blue light */
#define GREENINT  (5*60)    /* 5 mins gets green light */

#define NOTREPORTING (6*60)  /* Must report within 5 mins */

  xper = 0;
  if (station[strlen(station)-1]=='X') xper = 1;
  
  if (webmode) {
    if (xper) return;

    if (interv(last_record) < YELLOWINT) statball = "Yellow.gif";
    if (interv(last_record) < BLUEINT) statball = "Blue.gif";
    if (interv(last_record) < GREENINT) statball = "Green.gif";

    printf("<TR>");

    printf("<TD align=left> <img src=\"/pics/balls/%s\">  %s.%s.liss.org",
	   statball,station,network);

    /*printf("<TD align=right> %d ",con_port);*/

    printf("<TD align=right> %s ",uptime(startup));

    printf("<TD align=center>");

    if (interv(lastreport)>NOTREPORTING) 
      printf("<blink> <b> Process Not Reporting! </b> </blink>");
    else {

      if (strcmp(status,"NORMAL")==0) 
	printf("Ok");
      else 
	if (last_rec_error==ETIMEDOUT) 
	  printf("<b> Down or Unreachable </b> ");
	else
	  if (last_rec_error==EHOSTUNREACH)
	    printf("<b> Network Problem </b>");
	  else {
	    if (receiverconnected) 
	      printf("<b> Not Sending Data </b>");
	    else
	      printf("<b> Cannot Connect </b> ");
	  }
    }
    
    printf("<TD align=center> %3d/%-3d ",
	   retrain,connects);
    
    printf("<TD align=right> %s",uptime(last_record));
    
    printf("<TD align=right> %7ld <TD align=right> %7d",rectotal,seqno);
    
    printf("</tr>\n");
    
  } else {
    
    
    sprintf(work1,"%s %s",network,station);
    
    printf("%-8.8s",work1);
    
    printf("%s ",uptime(startup));
    
    sprintf(work1,"%s ",status);
    if (last_rec_error>0) 
      strcat(work1, strerror(last_rec_error));
    
    if (interv(lastreport)>NOTREPORTING) 
      strcpy(work1,"Process Not Reporting!");
    
    printf("%-30.30s",work1);
    
    printf("%3d/%-3d ",
	   retrain,connects);
    
    printf("%s",uptime(last_record));
    
    printf("%7ld%7d",rectotal,seqno);
    
    printf("\n");
    
  }
  
  fclose(inf);
  
}

int main()
{
  DIR *dirp;
  struct dirent *direntp;
  char *dcc;
  char spec[100],spec2[200];
  int i;
  
  webmode = 0;
  if (getenv("SERVER_NAME")) webmode = 1;
  
  if (webmode) setup_args();
  
  dcc = getenv("LISS_1LOGS");
#ifdef LISSSTATUS
  if (dcc==NULL)
    dcc = LISSSTATUS;
#endif
  if (dcc) 
    strcpy(spec,dcc);
  else {

    dcc = getenv("DCC_SETUPS");
    if (dcc==NULL)
      bombout(EXIT_ABORT,"DCC_SETUPS environment var not set\n");
    
    sprintf(spec,"%s/tel",dcc);
  }
  
  if (webmode) {

    time_t it = time(NULL);

    printf("<HTML>\n");
    printf("<HEAD>\n");
    printf("<TITLE>ASL DCC LISS Telemetry Status</TITLE>\n");
    printf("<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"60\">\n");
    printf("</HEAD>\n");
    printf("<body background=\"/pics/sandpaper.jpg\" bgcolor=\"#FFFFFF\">\n");
    printf("<center>");
    printf("<H1>ASL Remote Telemetry Status </h1>\n");
    printf("<img src=\"/pics/idbksm.gif\" width=\"80\" height=\"21\">\n");

    printf("<p>Generated: %s (Mountain time)</p>\n",ctime(&it));
    printf("<p><img src=\"/pics/line1.gif\" width=\"652\" height=\"8\"></p>");

    printf("<TABLE BORDER=1>\n");


    printf("<TR> <TH> Station/IP ");
    printf("<TH align=right> Uptime ");
    printf("<TH> Status ");
    printf("<TH> Retrain ");
    printf("<TH align=right> Last Rec ");
    printf("<TH align=right> Recs ");
    printf("<TH align=right> Conn </TR>\n");

  } else {
    
    printf("Station  Uptime   Status                        Retrain  Last Rec   Recs   Conn\n");
    
  }
  
  statnum = 0;
  dirp = opendir(spec);
  while ((direntp = readdir(dirp)) != NULL) {
    if (!WildMatch("forker_*_*.status", direntp->d_name)) continue;
    statfiles[statnum++] = strdup(direntp->d_name);
  }
  (void)closedir( dirp );

  qsort((void *) statfiles,
	statnum,
	(sizeof (char *)),
	compstr);

  for (i=0; i<statnum; i++) {
    sprintf(spec2,"%s/%s",spec,statfiles[i]);
    process_stats(spec2);
  }

  if (webmode) {
    printf("</TABLE>\n");

    printf("<p><img src=\"/pics/line1.gif\" width=\"652\" height=\"8\"></p>");

    printf("</center>\n");
    printf("<br><br><br>\n");

    printf("<br><b>Color Legend:</b>\n");
    printf("<blockquote>\n");
    printf("<img src=\"/pics/balls/Green.gif\"> Data received within last 5 minutes<br>\n");
    printf("<img src=\"/pics/balls/Blue.gif\"> Data received within last 4 hours<br>\n");
    printf("<img src=\"/pics/balls/Yellow.gif\"> Data received within last day<br>\n");
    printf("<img src=\"/pics/balls/Red.gif\"> No Data received within last day<br>\n");
    printf("</blockquote>\n");
    
    printf("<b> Port: </b>");
    printf("<blockquote>\n");
    printf("All LISS-1 ports are now 4000.\n");
    printf("Use the IP/DNS name for the station to connect.\n");
    printf("<p>For now, all stations send 512 byte seed records, except for the GTSN network (GT) which uses 256 bytes record lengths.\n");
    printf("</blockquote>\n");
	   
    printf("<b> Status: </b>");
    printf("<ul>\n");
    printf("<li><b>Ok</b> - Up and Transmitting Data<br>\n");
    printf("<li><b>Down or Unreachable </b> - Station is down or not otherwise answering network connections<br>\n");
    printf("<li><b>Network Problem </b> - Cannot reach station's network<br>\n");
    printf("<li><b>Not Sending Data</b> - Can connect to station, but it won't transmit data<br>\n");
    printf("<li><b>Cannot Connect </b> - Unable to connect (usually happens during a retry)<br>\n");
    printf("<li><b>Process Not Reporting!</b> - The telemetry repeater process for this station appears to be dead or broken<br>\n");
    printf("</ul>\n");
	   
    printf("<b> Retrain: </b>");
    printf("<blockquote>\n");
    printf("XXX/YYY - XXX is the number times the repeater has attempted to connect to the\n");
    printf("station since this repeater started.  YYY is the number of times repeater has sucessfully connected.\n");
    printf("</blockquote>\n");
	   
    printf("<b> Last Rec: </b>");
    printf("<blockquote>\n");
    printf("The time since the repeater saw a valid record from the station.  <i>Recent</i> is\n");
    printf("within the last minute, <i>Never</i> means data has not been seen since\n");
    printf("the repeater was started (see the <i>Uptime</i> field).\n");
    printf("</blockquote>\n");
	   
    printf("<b> Recs/Conn: </b>");
    printf("<blockquote>\n");
    printf("<i>Recs</i> is the number of records seen from the station since the\n");
    printf("repeater was started.  <i>Conn</i> is the number of records seen since\n");
    printf("the last connection was established.\n");
    printf("</blockquote>\n");
	   
    printf("</BODY>\n");
    printf("</HTML>\n");
  }

  exit(0);
}

